#include "reader.h"

static int	x_efd;
static uint32_t	x_next_day;

extern char	x_home_dir[256];

void dx_logfile_error(const char *fmt, ...)
{
    int		sz;
    uint32_t	today;
    time_t	t;
    char	*s;
    struct tm	*tp;
    va_list	args;
    char	tbuf[32];
    static char	emsg[2048];
    char	filename[1024];


    t  = time(NULL);
    tp = localtime(&t);
    sprintf(tbuf, "%04d-%02d-%02d %02d:%02d:%02d",
      tp->tm_year+1900,
      tp->tm_mon+1,
      tp->tm_mday,
      tp->tm_hour,
      tp->tm_min,
      tp->tm_sec
    );

    s = emsg;
    s += sprintf(s, "%s [%6d] ERROR ", tbuf, getpid());

    /* 2 bytes for \n & \0 */
    sz = 2048 - (s - emsg) - 2;

    va_start(args, fmt);
    vsnprintf(s, sz, fmt, args);
    va_end(args);

    sz = strlen(emsg);
    emsg[sz++] = '\n';
    emsg[sz]   = '\0';


    t  = time(NULL);
    tp = localtime(&t);
    today = tp->tm_year;
    today <<= 9;
    today |= tp->tm_mon<<5;
    today |= tp->tm_mday;

    if (today > x_next_day) {
        if (x_efd != -1) {
            close(x_efd);
            x_efd = -1;
         }

         x_next_day = today;
    }

    if (x_efd == -1) {
        /* create new runtime log file */
        sprintf(filename, "%s/log/log_%04d%02d%02d.txt",
            x_home_dir, tp->tm_year+1900, tp->tm_mon+1, tp->tm_mday);

        if ((x_efd = open(filename, O_CREAT|O_WRONLY|O_APPEND, 0644)) < 0) {
            x_efd = -1;
            return;
        }
    }


    while (1) {
        if (write(x_efd, emsg, sz) < 0) {
            if (errno == EINTR)
                continue;

            close(x_efd);
            x_efd  = -1;
        }

        break;
    }
}

void dx_close_logfile(void)
{
    if (x_efd != -1)
        close(x_efd);
    x_efd = -1;
}

