#include "reader.h"


int dx_create_listener(char *ipaddr, int port)
{
    int			sock;
    int			on;
    struct sockaddr_in	sa;


    sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock == -1) {
        dx_logfile_error("socket(): %s", strerror(errno));
        return (-1);
    }

    on = 1;
    if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
        (char *)&on, sizeof(on)) < 0) {
        dx_logfile_error("setsockopt(): %s", strerror(errno));
        close(sock);
        return (-1);
    }

    memset(&sa, 0, sizeof(sa));
    sa.sin_family      = AF_INET;
    sa.sin_addr.s_addr = inet_addr(ipaddr);
    sa.sin_port        = htons((short)port);

    if (bind(sock, (struct sockaddr *)&sa, sizeof(sa)) < 0) {
        dx_logfile_error("bind(): %s", strerror(errno));
        close(sock);
        return (-1);
    }

    if (listen(sock, 8) < 0) {
        dx_logfile_error("listen(): %s", strerror(errno));
        close(sock);
        return (-1);
    }

    return (sock);
}

int dx_accept_client(int sock)
{
    int			rc;
    socklen_t		socklen;
    fd_set		aset;
    struct sockaddr_in	peer;


    FD_ZERO(&aset);
    FD_SET(sock, &aset);

    if ((rc = select(sock+1, &aset, NULL, NULL, NULL)) < 0) {
        if (errno == EINTR)
            return (-1);
        dx_logfile_error("wait clicent select(): %s", strerror(errno));
        return (-1);
    }

    if (rc == 0) {
        dx_logfile_error("wait client select() return zero.");
        return (-1);
    }

    socklen = sizeof(peer);
    if ((rc = accept(sock, (struct sockaddr *)&peer, &socklen)) < 0) {
        dx_logfile_error("accept(): %s", strerror(errno));
        return (-1);
    }

    if (socklen == 0) {
        dx_logfile_error("socket length return zero");
        return (-1);
    }

    return (rc);
}

int dx_recv_data(int sock, uint8_t *buf, int buf_sz)
{
    int		rc;
    int		rsz;
    uint8_t	*pb;
    fd_set	rfd_set;


    if (sock < 0)
        return (-1);

    FD_ZERO(&rfd_set);
    FD_SET(sock, &rfd_set);

    while (((rc = select(sock+1, &rfd_set, NULL, NULL, NULL)) < 0) &&
          (errno == EINTR));
    if (rc < 0) {
        dx_logfile_error("wait data select(): %s", strerror(errno));
        return (-1);
    }

    if (rc == 0) {
        dx_logfile_error("wait data select() return zero.");
        return (-1);
    }

    rsz = buf_sz;
    pb  = buf;
    while (1) {
        if ((rc = recv(sock, pb, rsz, 0)) < 0) {
            if (errno == EINTR)
                continue;
            dx_logfile_error("recv(): %s", strerror(errno));
            return (-1);
        }

        if (rc == 0)
            return (0);

        pb  += rc;
        rsz -= rc;

        if ((int)(pb - buf) > 4)
            break;
    }

    /* get total size */
    rsz = dx_ntoh32(*(uint32_t *)buf) + 4 - (int)(pb - buf);

    while (rsz > 0) {
        if ((rc = recv(sock, pb, rsz, 0)) < 0) {
            if (errno == EINTR)
                continue;

            dx_logfile_error("second recv(): %s", strerror(errno));
            return (-1);
        }

        if (rc == 0)
            return (0);

        pb  += rc;
        rsz -= rc;
    }

    return ((int)(pb - buf));
}

int dx_send_data(int sock, uint8_t *buf, int dsz)
{
    int		rc;
    uint8_t	*pb;
    fd_set	wfd_set;


    if (sock < 0)
        return (-1);

    FD_ZERO(&wfd_set);
    FD_SET(sock, &wfd_set);

    pb = buf;
    while (dsz > 0) {
        if ((rc = select(sock+1, NULL, &wfd_set, NULL, NULL)) < 0) {
            if (errno == EINTR)
                continue;

            dx_logfile_error("send data select(): %s", strerror(errno));
            return (-1);
        }

        if ((rc = send(sock, pb, dsz, 0)) < 0) {
            if (errno == EINTR)
                continue;

            if (errno == EAGAIN)
                continue;

            dx_logfile_error("send(): %s", strerror(errno));
            return (-1);
        }

        pb  += rc;
        dsz -= rc;
    }

    return ((int)(pb - buf));
}

