#include "reader.h"

extern volatile sig_atomic_t	x_stop_server;
static int	x_fds[DX_MAX_FDS];

uint32_t dx_ntoh32(uint32_t n4)
{
    uint32_t	u4;
    uint8_t	*p;

    p = (uint8_t *)&n4;
    u4 = p[0];
    u4 = (u4<<8) + p[1];
    u4 = (u4<<8) + p[2];
    u4 = (u4<<8) + p[3];

    return (u4);
}

uint32_t dx_hton32(uint32_t h4)
{
    uint32_t	u4;
    uint8_t	*p;

    p = (uint8_t *)&u4;
    p[0] = (uint8_t)(h4>>24);
    p[1] = (uint8_t)(h4>>16);
    p[2] = (uint8_t)(h4>>8);
    p[3] = (uint8_t)(h4);

    return (u4);
}

uint64_t dx_ntoh64(uint64_t n8)
{
    uint64_t	u8;
    uint8_t	*p;

    p = (uint8_t *)&n8;
    u8 = p[0];
    u8 = (u8<<8) + p[1];
    u8 = (u8<<8) + p[2];
    u8 = (u8<<8) + p[3];
    u8 = (u8<<8) + p[4];
    u8 = (u8<<8) + p[5];
    u8 = (u8<<8) + p[6];
    u8 = (u8<<8) + p[7];

    return (u8);
}

uint64_t dx_hton64(uint64_t h8)
{
    uint64_t	u8;
    uint8_t	*p;

    p = (uint8_t *)&u8;
    p[0] = (uint8_t)(h8>>56);
    p[1] = (uint8_t)(h8>>48);
    p[2] = (uint8_t)(h8>>40);
    p[3] = (uint8_t)(h8>>32);
    p[4] = (uint8_t)(h8>>24);
    p[5] = (uint8_t)(h8>>16);
    p[6] = (uint8_t)(h8>>8);
    p[7] = (uint8_t)(h8);

    return (u8);
}

static int dx_send_error(int sock, uint8_t *buf, int bsz, int xerrno)
{
    int		mlen;
    uint32_t	tlen;
    uint8_t	*pb;

    pb = buf + 4;
    *((uint32_t *)pb) = dx_hton32(xerrno);
    pb += 4;
    snprintf((char *)pb, bsz-8, "%s", strerror(xerrno));

    mlen = strlen((char *)pb) + 4 + 1;
    tlen = mlen | 0x80000000;
    pb   = buf;
    *((uint32_t *)pb) = dx_hton32(tlen);
    if (dx_send_data(sock, buf, mlen+4) < 0) {
        dx_logfile_error("send error message: %s", strerror(errno));
        return (-1);
    }

    return (0);
}

int dx_worker(int sock)
{
    int		i;
    int		rc;
    int		buf_sz;
    int		cmd;
    int		fd;
    uint32_t	rsz;
    uint64_t	ofs;
    uint8_t	*pb;
    uint8_t	*buf;
    uint8_t	cbuf[32];


    buf_sz = DX_WORK_BUFSZ;
    if ((buf = malloc(buf_sz)) == NULL) {
        dx_logfile_error("allocate memory error: %s", strerror(errno));
        return (-1);
    }

    for (i=0; i<DX_MAX_FDS; i++) {
        x_fds[i] = -1;
    }

    x_stop_server = 0;
    while (x_stop_server == 0) {
        if ((rc = dx_recv_data(sock, buf, buf_sz)) < 0) {
            dx_logfile_error("receive data error: %s", strerror(errno));
            break;
        }

        /* client closed */
        if (rc == 0)
            break;

        pb = buf + 4;
        cmd = dx_ntoh32(*(uint32_t *)pb);

        if (cmd == DX_AGT_OPEN_FILE) {
            pb += 4;
            if ((fd = open((char *)pb, O_RDONLY)) < 0) {
                dx_logfile_error("open \"%s\" error.\n%s", (char *)pb, strerror(errno));
                dx_send_error(sock, buf, buf_sz, errno);
                continue;
            }

            pb = buf;
            *((uint32_t *)pb) = dx_hton32(4);
            pb += 4;
            *((uint32_t *)pb) = dx_hton32(fd);

            if (dx_send_data(sock, buf, 8) < 0) {
                dx_logfile_error("send file descriptor error: %s", strerror(errno));
                break;
            }

            for (i=0; i<DX_MAX_FDS; i++) {
                if (x_fds[i] == -1) {
                    x_fds[i] = fd;
                    break;
                }
            }
        } else if (cmd == DX_AGT_CLOSE_FILE) {
            pb += 4;
            fd = dx_ntoh32(*(uint32_t *)pb);
            while (((rc = close(fd)) < 0) && (errno == EINTR));
            if (rc < 0) {
                dx_logfile_error("close file error: %s", strerror(errno));
                dx_send_error(sock, buf, buf_sz, errno);
                break;
            }

            if (rc == 0)
                break;

            for (i=0; i<DX_MAX_FDS; i++) {
                if (x_fds[i] == fd) {
                    x_fds[i] = -1;
                    break;
                }
            }

            pb = buf;
            *((uint32_t *)pb) = dx_hton32(0);

            if (dx_send_data(sock, buf, 4) < 0) {
                dx_logfile_error("send file descriptor error: %s", strerror(errno));
                break;
            }
        } else if (cmd == DX_AGT_READ_DATA) {
            /* fd + rsz + offset */
            pb += 4;
            fd  = dx_ntoh32(*(uint32_t *)pb);
            pb += 4;
            rsz = dx_ntoh32(*(uint32_t *)pb);
            pb += 4;
            ofs = dx_ntoh64(*(uint64_t *)pb);

            if (rsz > buf_sz)
                rsz = buf_sz;
            pb = buf;
            while (rsz > 0) {
                if ((rc = pread(fd, pb, rsz, ofs)) < 0) {
                    if (errno == EINTR)
                        continue;
                    dx_logfile_error("read file error: %s\n%fd=%d, size=%d, offset=%llu",
                        strerror(errno), fd, rsz, ofs);
                    dx_send_error(sock, buf, buf_sz, errno);
                    break;
                }

                if (rc == 0)
                    break;

                pb  += rc;
                rsz -= rc;
                ofs += rc;
            }

            if (rc < 0)
                continue;

            rsz = (uint32_t)(pb - buf);
            *((uint32_t *)cbuf) = dx_hton32(rsz);
            if (dx_send_data(sock, cbuf, 4) < 0) {
                dx_logfile_error("send data length error: %s", strerror(errno));
                break;
            }

            if (dx_send_data(sock, buf, rsz) < 0) {
                dx_logfile_error("send data error: %s", strerror(errno));
                break;
            }
        } else {
            dx_logfile_error("invalid command: %d", cmd);
        }
    }

    for (i=0; i<DX_MAX_FDS; i++) {
        if (x_fds[i] != -1) {
            close(x_fds[i]);
        }
    }

    return (0);
}
